import json
import os.path as path
import unittest


class TestTruffleTemplate(unittest.TestCase):
    def setUp(self):
        """Call before every test case."""
        text_file_path = path.dirname(path.abspath(__file__)) + '/../templates/truffle-config.txt'
        self.truffle_config_f = open(text_file_path, "r")
        self.truffle_config = self.truffle_config_f.read()

    def tearDown(self):
        """Call after every test case."""
        self.truffle_config_f.close()

    def testPort(self):
        "Check if there is the PORT option"
        assert self.truffle_config.replace('<PORT>', 'Found it')

    def testFrom(self):
        "Check if there is the From option"
        assert self.truffle_config.replace('<FROM>', 'Found it')


class TestGenesis(unittest.TestCase):
    def setUp(self):
        """Call before every test case."""
        text_file_path = path.dirname(path.abspath(__file__)) + '/../templates/genesis.json'
        self.genesis_f = open(text_file_path, "r")
        self.genesis = self.genesis_f.read()

    def tearDown(self):
        """Call after every test case."""
        self.genesis_f.close()

    def testFrom(self):
        "Check if the json of genesis is as should"
        string_genesis = "{  \"nonce\": \"0x0000000000000042\",  \"timestamp\": \"0x0\",  \"parentHash\": " \
                         "\"0x0000000000000000000000000000000000000000000000000000000000000000\",  \"extraData\": " \
                         "\"0x00\",  \"gasLimit\": \"0x8000000\",  \"difficulty\": \"0x400\",  \"mixhash\": " \
                         "\"0x0000000000000000000000000000000000000000000000000000000000000000\",  \"coinbase\": " \
                         "\"0x3333333333333333333333333333333333333333\",  \"alloc\": {},  \"config\": {    " \
                         "\"chainId\": 4777,    \"homesteadBlock\": 0,    \"eip150Block\": 0," \
                         "    \"eip150Hash\": " \
                         "\"0x0000000000000000000000000000000000000000000000000000000000000000\"," \
                         "    \"eip155Block\": 0,    \"eip158Block\": 0,    \"byzantiumBlock\": 0," \
                         "    \"constantinopleBlock\": 0,    \"petersburgBlock\": 0,    \"ethash\": {}, " \
                         "\"isQuorum\": true," \
                         "\"maxCodeSize\": 128," \
                         "\"txnSizeLimit\": 128  }} "

        assert json.loads(self.genesis) == json.loads(string_genesis)


if __name__ == '__main__':
    unittest.main()



import unittest

from python_geth.tests import templates_test, node_test


def test_suite():
    """Add all your tests here"""
    suite = unittest.TestLoader().loadTestsFromModule(templates_test)
    suite.addTests(unittest.TestLoader().loadTestsFromModule(node_test))
    return suite


if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(test_suite())



import json
import os


class ContractInterface:
    def __init__(self, w3, datadir):
        self.w3 = w3
        self.datadir = datadir

    @staticmethod
    def __get__contract_names(contract_file):
        r = []
        with open(contract_file) as f:
            lines = f.read().splitlines()
            matching = [s for s in lines if "contract" in s]

            for item in matching:
                item = item.split(" ")
                item = item[1]
                r.append(item)
        return r

    def __amend_migrations(self, contract_names, constructor_params):
        if not isinstance(constructor_params, list) and constructor_params is not None:
            raise TypeError("Params of constructor should be passed as list")
        if constructor_params is not None and len(contract_names) != len(constructor_params):
            raise Warning(
                'contract_names and constructor_params should have the same length! If a contract does not need '
                'constructor params pass an empty string in its place')

        with open('{}/migrations/1_initial_migration.js'.format(self.datadir), 'r+') as migrations_f:
            migrations = migrations_f.read()
            migrations = migrations.split(";")
            new_deployer = ''
            new_const = ''

            for i in range(len(contract_names)):
                contract_name = contract_names[i]

                if constructor_params is None:
                    new_deployer += " deployer.deploy({0}) ".format(contract_name)
                else:
                    constructor_param = constructor_params[i]
                    new_deployer += "deployer.deploy({0},\"{1}\"); ".format(contract_name, constructor_param)
                new_const += "const {0} = artifacts.require(\"{0}\"); ".format(contract_name)

            indices = [i for i, s in enumerate(migrations) if 'artifacts.require' in s]
            migrations.insert(indices[0], new_const)

            indices = [i for i, s in enumerate(migrations) if 'deployer.deploy' in s]

            migrations.insert(indices[0] + 1, new_deployer)

            migrations = ";".join(migrations)
            migrations_f.seek(0)
            migrations_f.write(migrations)
            migrations_f.truncate()

    def set_default_account(self):
        self.w3.eth.defaultAccount = self.w3.eth.accounts[0]

    def get_contract_from_source(self, source, networkid='1900'):
        with open(source) as f:
            data = json.load(f)
            return self.w3.eth.contract(address=data['networks'][networkid]['address'],
                                        abi=data['abi'])

    def deploy_contract(self, contract_file, constructor_params=None, networkid='1900', default_account=True):
        """
        Make sure your account is unlocked!

        :param constructor_params:
        :param networkid:
        :param contract_file:
        :return:
        """
        if not os.path.exists(contract_file):
            raise FileNotFoundError("Cannot find the file specified {}".format(contract_file))
        else:
            os.system('cp {} {}/contracts'.format(contract_file, self.datadir))

            contract_names = self.__get__contract_names(contract_file)

            self.__amend_migrations(contract_names, constructor_params)

            command = "cd {}/contracts && npx truffle migrate".format(self.datadir)
            try:
                os.system(command)
            except Exception as e:
                print('Compilation failed: \n {}'.format(e))

            if default_account:
                self.set_default_account()

            r = []
            for contract in contract_names:
                with open('{}/build/contracts/{}.json'.format(self.datadir, contract)) as f:
                    data = json.load(f)
                    r.append(self.w3.eth.contract(address=data['networks'][networkid]['address'],
                                                  abi=data['abi']))

            return tuple(r)



import atexit
import hashlib
import json
import os
import subprocess
import time
from datetime import datetime
from shutil import which, copyfile
from subprocess import Popen

from web3 import Web3, HTTPProvider


class Node:
    def __init__(self, datadir, port=30303, rpcport=8000, name="Node01", netowrk_id=1900, genesis_file=''):
        """
        Create a Node object

        :param datadir: Absolute path to a directory to be used for data of the node
        :type datadir: str
        :param port: Geth port
        :type port: int
        :param rpcport: Geth RpcPort (old: http port)
        :type rpcport: int
        :param name: Name of your node
        :type name:str
        :param netowrk_id: The id of your ETH network
        :type netowrk_id: int
        :param genesis_file: Absolute path to genesis file
        :type genesis_file: str
        """

        self._check_for_geth()
        self.port = port
        self.rpcport = rpcport
        self.datadir = datadir
        self.name = name
        self.network_id = netowrk_id
        self.process = None
        self.http = "http://127.0.0.1:{}".format(rpcport)
        self.genesisFile = genesis_file if genesis_file != '' else None
        self._create_node()
        self.w3 = None

    def start_node(self):
        """
        Starts the node in a process
        :return: null
        :rtype: null
        """
        if os.name == 'nt':
            command = "geth --identity {0} -cache=1024 --syncmode \"fast\" --http --http.port {1} --http.corsdomain \"*\" " \
                      "--datadir \"{2}\" --port " \
                      "{3} --nodiscover --http.api  \"eth,net,web3,personal,miner,admin\" --networkid {4} --nat " \
                      "\"any\" --ipcdisable --allow-insecure-unlock ".format(str(self.name), str(self.rpcport),
                                                                             str(self.datadir),
                                                                             str(self.port), str(self.network_id))

            self.process = Popen(command)

        else:
            command = "exec geth --identity {0}  -cache=1024 --syncmode \"fast\" --http --http.port {1} --http.corsdomain \"*\" --datadir \"{2}\" " \
                      "--port " \
                      "{3} --nodiscover --http.api  \"eth,net,web3,personal,miner,admin\" --networkid {4} --nat " \
                      "\"any\" --ipcdisable --allow-insecure-unlock ".format(str(self.name), str(self.rpcport),
                                                                             str(self.datadir),
                                                                             str(self.port), str(self.network_id))

            self.process = Popen(command, stdout=subprocess.PIPE, shell=True)

        self.w3 = Web3(HTTPProvider('http://127.0.0.1:{}'.format(self.rpcport)))
        counter = 0

        # This is a brute force way of waiting for an async call
        while not self.w3.isConnected() or counter >= 5:
            time.sleep(1)
            counter += 1
        if self.w3.isConnected():
            print('STARTED PID: ', self.process.pid)
            atexit.register(self.stop_node)
        else:
            self.stop_node()

    def stop_node(self):
        """
        Stops the nodes process
        """
        try:
            print("Killing process")
            self.process.kill()
            self.process.wait()
        except Exception as e:
            print('PLease report this!')
            print(e)
            if os.name == 'nt':
                print("Killing process failed: Forcing!")
                os.system("taskkill /im geth.exe /f")

    @staticmethod
    def _check_for_geth():
        if not which("geth") is not None:
            print("Geth needs to be installed and added to path! ")
            print("Exiting!")
            exit(1)

    @staticmethod
    def _check_for_npm():
        if not which("npx") is not None:
            print("Npm needs to be installed and added to path! ")
            print("Exiting!")
            exit(1)

    def _create_node(self):
        """
        Wrapper method called on __init__ to create a geth node.
        """
        try:
            os.mkdir("{}".format(self.datadir))
        except FileExistsError:
            print("WARNING: The folder for the node exists, but will proceed to use it!")

        if self.genesisFile is None:
            try:
                os.mkdir("{}/config".format(self.datadir))
            except FileExistsError:
                print("WARNING: The folder for the config exists, but will proceed to use it!")

            now = datetime.now()
            date_time = now.strftime("%m/%d/%Y, %H:%M:%S")
            pass_path = "{0}/pass_first.txt".format(self.datadir)
            with open(pass_path, "w") as pass_file:
                pass_file.write(hashlib.sha256(date_time.encode('utf-8')).hexdigest())

            os.system("geth --datadir \"{0}\" account new --password \"{1}\"".format(self.datadir, pass_path))

            accounts_adresses = {}

            for root, dirs, files in os.walk("{}/keystore".format(self.datadir)):
                for file in files:
                    with open(os.path.join(root, file)) as account:
                        data = json.load(account)
                        accounts_adresses[data['address']] = {"balance": "1000000000000000000"}
            fn = os.path.join(os.path.dirname(__file__), 'templates/genesis.json')

            with open(fn) as template:
                data = json.load(template)
                data['alloc'] = accounts_adresses
                with open("{}/config/genesis.json".format(self.datadir), 'w+') as write_file:
                    json.dump(data, write_file, indent=4)

            os.system("geth --datadir \"{0}\" init \"{0}/config/genesis.json\" ".format(self.datadir))
        else:
            os.system("geth --datadir \"{0}\" init \"{1}\" ".format(self.datadir, self.genesisFile))

    def add_node(self, enode, localhost=True):
        """
        Add a node to the chain.

        :param enode: The enode address of the node to be added
        :type enode: string
        :param localhost: whether the node is running on localhost or not
        :type localhost: bool
        :return: success indicator
        :rtype: bool
        """
        if localhost:
            enode_address = enode.split("@")
            ending = enode_address[1].split("?")
            ip = ending[0].split(":")
            ip[0] = "[::]"
            ip = ":".join(ip)
            enode_address = enode_address[0] + "@" + ip + "?" + ending[1]
        else:
            enode_address = enode
        count = self.w3.net.peer_count
        self.w3.geth.admin.add_peer(enode_address)
        if count + 1 == self.w3.net.peer_count:
            return True
        else:
            return False

    def add_foreign_account(self, key, name):
        copyfile(key, "{}/keystore/{}".format(self.datadir, name))

    def get_first_account(self):
        """
        Get the first account that gets created as part of the create node function.
        This account can be used to interface with geth for easy contract deployment.

        :return: string of account number, string of password
        :rtype: str,str
        """
        account = self.w3.eth.accounts[0]
        if os.path.exists("{0}/pass_first.txt".format(self.datadir)):
            with open("{0}/pass_first.txt".format(self.datadir), "r") as pass_file:
                passwd = pass_file.read()
        else:
            passwd = ''

        return account, passwd.strip()

    def configure_truffle(self, config_file=None):
        """
        Set up default truffle to be ready to deploy contracts. This method needs to be called before
        creating contract interfaces.

        :param config_file: optional absolute path to config file to be supplied in case of specific config
        :type config_file: str

        """
        if config_file is not None:
            template_file = config_file
        else:
            template_file = os.path.join(os.path.dirname(__file__), 'templates/truffle-config.txt')

        os.system("cd {} && npx truffle init".format(self.datadir))
        os.system("rm {}/truffle-config.js".format(self.datadir))
        with open(template_file, 'r') as template_f:
            template = template_f.read()
            template = template.replace('<PORT>', str(self.rpcport))
            template = template.replace('<FROM>', "\"{}\"".format(self.get_first_account()[0]))
            with open('{}/truffle-config.js'.format(self.datadir), 'w+') as original_config_f:
                original_config_f.write(template)



import os
import unittest

from python_geth.node import Node


class TestNodeClass(unittest.TestCase):
    def setUp(self):
        """Call before every test case."""
        self.node = Node(datadir="node01_test", port=30303, rpcport=8000, name="Node01")
        self.node.start_node()

    def tearDown(self):
        """Call after every test case."""
        self.node.stop_node()
        os.system('rm -rf node01_test')

    def testW3(self):
        assert self.node.w3.isConnected()


if __name__ == '__main__':
    unittest.main()
